package EFS::CLI::AutoRelease;

use strict;
use warnings;
use English;

use base q{EFS::CLI};
use base q{EFS::DBI};

use Date::Format qw( time2str );
use Params::Validate qw( :all );

sub create {

    my $class = shift;

    my (%args) = validate( @_, {
        %EFS::CLI::validate_common,
        metaproj => { type       => SCALAR },
        project  => { type       => SCALAR },
    });

    my $project_rs = $class->get_resultset( q{Project} );
    my $project = $project_rs->retrieve_bynames_assert(
        metaproj => $args{metaproj},
        name     => $args{project},
    );

    $project->assert_auto_project;

    return EFS::RPC->execute(
        cell   => $project->cell,
        class  => $class,
        method => q{create_rpc},
        args   => [ %args ],
    );

}

sub create_rpc {

    my $class = shift;
    my %args  = @_;

    my $project_rs = $class->get_resultset( q{Project} );
    my $project = $project_rs->retrieve_bynames_assert(
        metaproj => $args{metaproj},
        name     => $args{project},
    );

    my $history_rs = $class->get_resultset( q{History} );
    my $history = $history_rs->retrieve_assert( id => $args{history} );

    my $lock = EFS::DirLock->new({
        directory => $project->get_path(q{dev}),
        retry     => 1,
        command   => $history->request,
    });

    my $releasealias_rs = $class->get_resultset( q{ReleaseAlias} );
    my $nextalias = $releasealias_rs->retrieve_bynames(
        metaproj => $args{metaproj},
        project  => $args{project},
        name     => q{next},
    );

    my $release_name = q{};

    if ( $nextalias ) {
        $release_name = $nextalias->target->name;
    }
    else {
        $release_name = time2str( q{%Y%m%d-%H%M%S}, time, q{GMT} );
    }

    my $release = EFS::CLI::Release->create_rpc(
        history         => $args{history},
        metaproj        => $args{metaproj},
        project         => $args{project},
        name            => $release_name,
        _project_locked => 1,
    );

    my $install = EFS::CLI::Install->create_rpc(
        history  => $args{history},
        metaproj => $args{metaproj},
        project  => $args{project},
        release  => $release_name,
        name     => q{common},
    );

    my $alias = EFS::CLI::ReleaseAlias->create_rpc(
        history         => $args{history},
        metaproj        => $args{metaproj},
        project         => $args{project},
        name            => q{next},
        target          => $release_name,
        _project_locked => 1,
    );

    STDOUT->print( qq{Created autorelease $release\n} );

    return $release;

}

sub dist {

    my $class = shift;

    my (%args) = validate( @_, {
        %EFS::CLI::validate_common,
        metaproj => { type       => SCALAR },
        project  => { type       => SCALAR },
    });

    my $project_rs = $class->get_resultset( q{Project} );
    my $project = $project_rs->retrieve_bynames_assert(
        metaproj => $args{metaproj},
        name     => $args{project},
    );

    $project->assert_auto_project;

    return EFS::RPC->execute(
        cell   => $project->cell,
        class  => $class,
        method => q{dist_rpc},
        args   => [ %args ],
    );

}

sub dist_rpc {

    my $class = shift;
    my %args  = @_;

    my $project_rs = $class->get_resultset( q{Project} );
    my $project = $project_rs->retrieve_bynames_assert(
        metaproj => $args{metaproj},
        name     => $args{project},
    );

    my $history_rs = $class->get_resultset( q{History} );
    my $history = $history_rs->retrieve_assert( id => $args{history} );

    my $lock = EFS::DirLock->new({
        directory => $project->get_path(q{dev}),
        retry     => 1,
        command   => $history->request,
    });

    my $releasealias_rs = $project->get_resultset( q{ReleaseAlias} );
    my $nextalias = $releasealias_rs->retrieve_bynames_assert(
        metaproj => $args{metaproj},
        project  => $args{project},
        name     => q{next},
    );

    my $release      = $nextalias->target;
    my $release_name = $release->name;

    EFS::CLI::Release->checkpoint_rpc(
        history  => $args{history},
        metaproj => $args{metaproj},
        project  => $args{project},
        name     => $release_name,
    );

    EFS::CLI::ReleaseAlias->destroy_rpc(
        history         => $args{history},
        metaproj        => $args{metaproj},
        project         => $args{project},
        name            => q{next},
        _project_locked => 1,
    );

    EFS::CLI::Release->dist_rpc(
        history  => $args{history},
        metaproj => $args{metaproj},
        project  => $args{project},
        name     => $release_name,
        celltype => q{dev},
        global   => 1,
    );

    EFS::CLI::Release->setstage_rpc(
        history  => $args{history},
        metaproj => $args{metaproj},
        project  => $args{project},
        name     => $release_name,
        stage    => q{prod},
    );

    EFS::CLI::Release->dist_rpc(
        history  => $args{history},
        metaproj => $args{metaproj},
        project  => $args{project},
        name     => $release_name,
        celltype => q{all},
        global   => 1,
    );

    my ($releasealias)  = $releasealias_rs->retrieve_bynames(
        metaproj => $args{metaproj},
        project  => $args{project},
        name     => q{current},
    );

    my $previous_release;

    if ( $releasealias ) {
        $previous_release = $releasealias->target;
    }

    EFS::CLI::ReleaseAlias->create_rpc(
        history         => $args{history},
        metaproj        => $args{metaproj},
        project         => $args{project},
        name            => q{current},
        target          => $release_name,
        _project_locked => 1,
    );

    EFS::CLI::ReleaseAlias->dist_rpc(
        history         => $args{history},
        metaproj        => $args{metaproj},
        project         => $args{project},
        name            => q{current},
        celltype        => q{all},
        global          => 1,
        _project_locked => 1,
    );

    if ( $previous_release ) {

        my $previous_stage = $previous_release->stage;

        my $newstage =
            $previous_release->has_prod_stages ? q{deprecated} : q{rejected};

        EFS::CLI::Release->setstage(
            history  => $args{history},
            metaproj => $args{metaproj},
            project  => $args{project},
            name     => $previous_release->name,
            stage    => $newstage,
        );

        EFS::CLI::Release->destroy(
            history         => $args{history},
            metaproj        => $args{metaproj},
            project         => $args{project},
            name            => $previous_release->name,
            _project_locked => 1,
        );

    }

    STDOUT->print( qq{Disted autorelease $release\n} );

    return 1;

}

sub revert {

    my $class = shift;

    my (%args) = validate( @_, {
        %EFS::CLI::validate_common,
        metaproj  => { type     => SCALAR },
        project   => { type     => SCALAR },
        name      => { type     => SCALAR,
                       optional => 1 },
        nodestroy => { type     => SCALAR,
                       optional => 1 },
        force     => { type     => SCALAR,
                       optional => 1 },
    });

    my $project_rs = $class->get_resultset( q{Project} );
    my $project = $project_rs->retrieve_bynames_assert(
        metaproj => $args{metaproj},
        name     => $args{project},
    );

    $project->assert_auto_project;

    return EFS::RPC->execute(
        cell   => $project->cell,
        class  => $class,
        method => q{revert_rpc},
        args   => [ %args ],
    );

}

sub revert_rpc {

    my $class = shift;
    my %args  = @_;

    my $project_rs = $class->get_resultset( q{Project} );
    my $project = $project_rs->retrieve_bynames_assert(
        metaproj => $args{metaproj},
        name     => $args{project},
    );

    my $history_rs = $class->get_resultset( q{History} );
    my $history = $history_rs->retrieve_assert( id => $args{history} );

    my $lock = EFS::DirLock->new({
        directory => $project->get_path(q{dev}),
        retry     => 1,
        command   => $history->request,
    });

    my $previous;

    if ( $args{name} ) {

        my $release_rs = $class->get_resultset( q{Release} );
        $previous = $release_rs->retrieve_bynames_assert(
            metaproj  => $args{metaproj},
            project   => $args{project},
            name      => $args{name},
            destroyed => 1,
        );

        if ( $previous->reverted ) {
            if ( not $args{force} ) {
                die(
                    qq{Unable to revert to a previously reverted autorelease\n},
                    qq{To revert to $previous, use --force\n},
                );
            }
            else {
                $previous->delete_attribute( name => q{reverted} );
            }
        }

    }
    else {

        foreach my $destroyed ( reverse sort $project->destroyed_releases ) {
            next if $destroyed->reverted;
            $previous = $destroyed;
            last;
        }

        if ( not $previous ) {
            die qq{Unable to determine previous release to revert\n};
        }

    }

    EFS::CLI::Release->recover_rpc(
        history         => $args{history},
        metaproj        => $args{metaproj},
        project         => $args{project},
        name            => $previous->name,
        _project_locked => 1,
    );

    my $releasealias_rs = $class->get_resultset( q{ReleaseAlias} );
    my $current = $releasealias_rs->retrieve_bynames_assert(
        metaproj => $args{metaproj},
        project  => $args{project},
        name     => q{current},
    );

    my $destroy = $current->target;

    EFS::CLI::ReleaseAlias->create_rpc(
        history         => $args{history},
        metaproj        => $args{metaproj},
        project         => $args{project},
        name            => q{current},
        target          => $previous->name,
        _project_locked => 1,
    );

    EFS::CLI::ReleaseAlias->dist_rpc(
        history         => $args{history},
        metaproj        => $args{metaproj},
        project         => $args{project},
        name            => q{current},
        celltype        => q{all},
        global          => 1,
        _project_locked => 1,
    );

    if ( $args{nodestroy} ) {
        return 1;
    }

    my $newstage = $destroy->has_prod_stages ? q{deprecated} : q{rejected};

    EFS::CLI::Release->setstage(
        history  => $args{history},
        metaproj => $args{metaproj},
        project  => $args{project},
        name     => $destroy->name,
        stage    => $newstage,
    );

    EFS::CLI::Release->destroy(
        history         => $args{history},
        metaproj        => $args{metaproj},
        project         => $args{project},
        name            => $destroy->name,
        _project_locked => 1,
    );

    $destroy->set_attribute(
        name  => q{reverted},
        value => 1,
    );

    STDOUT->print( qq{Reverted to autorelease $previous\n} );

    return 1;

}

1;

# Local Variables:
#   mode: cperl
#   cperl-indent-level: 4
#   fill-column: 78
# End:
# vim: expandtab shiftwidth=4:
