package EFS::CLI::Database;

use strict;
use warnings;
use English;

use base q{EFS::CLI};
use base q{EFS::DBI};

use Params::Validate qw( :all );
use FreezeThaw qw(cmpStr);
use File::Slurp;
use Date::Format qw( time2str );
use File::Basename qw( dirname );
use Readonly;

use EFS::FQDN qw(fqdn);

Readonly my $BACKUP_RETAIN => 5;

sub backup {

    my $class = shift;

    my (%args) = validate( @_, {
        %EFS::CLI::validate_common,
    });

    my $parameter_rs     = $class->get_resultset( q{Parameter} );
    my $database_primary = $parameter_rs->configured( q{database_primary} ) ||
        die qq{Unable to backup database: No database_primary parameter\n};

    my $efsserver_rs = $class->get_resultset( q{EFSServer} );
    my $efsserver    = $efsserver_rs->retrieve_bynames_assert( name => $database_primary );

    return EFS::RPC->execute_on(
        efsservers => [ $efsserver ],
        class      => $class,
        method     => q{backup_rpc},
        args       => [%args],
    );

}

sub backup_rpc {

    my $class = shift;
    my %args  = @_;

    my $dbvendor      = $class->get_db_vendor;
    my $backup_method = qq{backup_$dbvendor};

    if ( not $class->can( $backup_method ) ) {
        die qq{Database backups are not support for vendor $dbvendor\n};
    }

    my $backupdir = q{/var/efs/db/backup};

    EFS::Util->MakeDir( path => $backupdir );

    my $lock = EFS::DirLock->new({
        directory => $backupdir,
        command   => q{efs backup database},
    });

    $class->$backup_method( $backupdir );

    my $io = IO::Dir->new( $backupdir ) || die qq{Unable to opendir $backupdir: $ERRNO\n};

    my @backups = ();

    foreach ( reverse sort $io->read ) {
        next if not m{\.bz2$}ms;
        push @backups, qq{$backupdir/$_};
    }

    my $parameter_rs  = $class->get_resultset( q{Parameter} );
    my $backup_retain =
        $parameter_rs->configured( q{database_backup_retain} ) || $BACKUP_RETAIN;

    splice( @backups, 0, $backup_retain );

    if ( @backups ) {
        STDOUT->print( qq{Removing these old backups:\n}, map { qq{$_\n} } @backups );
        foreach my $backup ( @backups ) {
            unlink $backup or die qq{Unable to unlink $backup: $ERRNO\n};
        }
    }

    my $database_secondary = $parameter_rs->configured( q{database_secondary} );
    # XXX: Need a message here?  Quiet or verbose?
    return 1 if not $database_secondary;

    my $efsserver_rs = $class->get_resultset( q{EFSServer} );
    my $efsserver = $efsserver_rs->retrieve_bynames_assert( name => $database_secondary );

    EFS::RPC->execute_on(
        efsservers => [ $efsserver ],
        class      => q{EFS::Util},
        method     => q{MakeDir},
        args       => [ path => dirname( $backupdir ) ],
    );

    EFS::Rsync->new({
        dest_efsserver => $efsserver,
        src_path       => $backupdir,
        dest_path      => $backupdir,
    });

    STDOUT->print( qq{Copied $dbvendor backup data to $database_secondary\n} );
    return 1;

}

sub backup_mysql {

    my $class     = shift;
    my $backupdir = shift;

    my $config = EFS::Config::Server->new;
    my $user   = $config->get_value( q{database}, q{dbouser} ) ||
        die qq{Unable to backup database: no dbouser defined\n};

    my $file = qq{/var/efs/db/passwd/$user};

    die qq{No such file: $file\n} if not -f $file;

    my $passwd = read_file( $file );
    chomp $passwd;

    my $backupfile = time2str( q{efs-%Y%m%d-%H%M%S}, time, q{GMT} ) . q{.sql};

    my $fqdn = fqdn;

    my $shell = join(
        q{ },
        qq{mysqldump --single-transaction --host=$fqdn},
        qq{--user=$user --password=$passwd efs > $backupdir/$backupfile},
    );

    system $shell;
    $CHILD_ERROR and die qq{Unable to backup database\n};

    system qq{bzip2 $backupdir/$backupfile};
    $CHILD_ERROR and die qq{Unable to compress $backupdir/$backupfile\n};

    STDOUT->print( qq{Database dumped to $backupfile.bz2\n} );
    return 1;

}

sub dumpconfig {

    my $class = shift;

    my (%args) = validate( @_, {
        %EFS::CLI::validate_common,
        %EFS::CLI::validate_dumpconfig_args,
        file => { type => SCALAR, optional => 1 },
    });

    if ( $args{file} ) {

        return $class->dumpconfig_file( $args{file} );

    }
    else {

        my $project_rs = $class->get_resultset(q{Project});

        my $project = $project_rs->retrieve_bynames_assert(
            metaproj => q{efs},
            name     => q{config-database},
        );

        return EFS::RPC->execute(
            cell   => $project->cell,
            class  => $class,
            method => q{dumpconfig_rpc},
            args   => [%args],
        );

    }

}

sub dumpconfig_rpc {

    my $class = shift;

    my %args = @_;

    my $history_rs = $class->get_resultset(q{History});
    my $history = $history_rs->retrieve_assert( id => $args{history} );

    my $project_rs = $class->get_resultset(q{Project});
    my $project = $project_rs->retrieve_bynames_assert(
        metaproj => q{efs},
        name     => q{config-database},
    );

    my $project_dir = $project->get_path(q{dev});

    my $release = EFS::CLI::AutoRelease->create_rpc(
        history  => $history->id,
        metaproj => q{efs},
        project  => q{config-database},
    );

    my ($install) = $release->installs({ name => q{common} });
    my $install_dir = $install->get_path(q{dev});

    my $dbold = qq{$project_dir/current/install/common/efs.db};
    my $dbnew = qq{$install_dir/efs.db};

    system(qq{cp $dbold $dbnew});
    if ( $CHILD_ERROR ) {
        die qq{Unable to copy $dbold to $dbnew\n};
    }

    $class->dumpconfig_file( $dbnew );

    STDOUT->print( qq{Extracted all database tables\n} );

    if ( $args{nodist} ) {
        STDOUT->print( qq{Distribution suppressed (-nodist)\n} );
        return 1;
    }

    EFS::CLI::AutoRelease->dist_rpc(
        history  => $history->id,
        metaproj => q{efs},
        project  => q{config-database},
    );

    return 1;

}

sub dumpconfig_file {

    my $class = shift;
    my $file = shift;

    STDOUT->print( qq{Extracting database to $file\n} );

    my $batchcount = 1000;

    my $schema_in = $class->get_schema;

    my $schema_out = $class->connect(
        q{dbi:SQLite:dbname=:memory:},q{sqlite},q{sqlite},
        {
            AutoCommit => 1,
            RaiseError => 1,
        },
        {
            on_connect_call => q{datetime_setup},
            quote_char      => q{"},
            name_sep        => q{.},
        },

    );

    my $dbh_in  = $schema_in->storage->dbh;
    my $dbh_out = $schema_out->storage->dbh;

    my $prefix = $class->get_db_prefix || q{efs};

    $dbh_out->do(qq{ATTACH DATABASE "$file" AS $prefix});
    $dbh_out->{AutoCommit} = 0;

    my %skip = map { $_ => 1 }
        qw( History DistStats RsyncStats );

    my $time = time;

    foreach my $source ( sort $class->get_schema->sources ) {

        if ( $skip{ $source } ) {
            STDOUT->print( qq{Skipping source $source\n} );
            next;
        }

        STDOUT->print( qq{Extracting database table for $source\n} );

        my $starttime = time;

        my $table_in =
            $schema_in->get_table_string( $source, $class->get_db_quote_char );
        my $table_out = $schema_out->get_table_string( $source, q{"} );

        my $sth_in_read = $dbh_in->prepare(qq{select * from $table_in});
        $sth_in_read->execute;

        my %data_in = ();
        while ( my $row = $sth_in_read->fetchrow_hashref ) {
            $data_in{ $row->{id} } = $row;
        }

        my $sth_out_read = $dbh_out->prepare(qq{select * from $table_out});
        $sth_out_read->execute;

        my %data_out = ();
        while ( my $row = $sth_out_read->fetchrow_hashref ) {
            $data_out{ $row->{id} } = $row;
        }

        if ( not %data_in and not %data_out ) {
            STDOUT->print(qq{No data to process for $source\n});
            next;
        }

        my $sth_out_delete =
            $dbh_out->prepare_cached(qq{delete from $table_out where id = ?});

        my $deleted  = 0;
        my $updated  = 0;
        my $inserted = 0;

        my $count = 0;

        foreach my $id ( sort keys %data_out ) {
            if ( not exists $data_in{ $id } ) {
                $sth_out_delete->execute($id);
                $deleted++;
                $count++;
                if ( ($count % $batchcount) == 0 ) {
                    $dbh_out->commit;
                }
            }
        }

        $dbh_out->commit;

        my $first_row;

        if ( not %data_in ) {
            ($first_row) = values %data_out;
        }
        else {
            ($first_row) = values %data_in;
        }

        my @fields = sort keys %{ $first_row };

        my $sql = sprintf q{insert into %s (%s) values (%s)},
            $table_out,
                join(q{,}, map { qq{"$_"} } @fields),
                    join(q{,}, (q{?})x@fields);

        my $sth_out_insert = $dbh_out->prepare_cached($sql);

        $count = 0;

        foreach my $id ( sort keys %data_in ) {

            my @values = @{ $data_in{ $id } }{ @fields };

            if ( not exists $data_out{$id} ) {
                $sth_out_insert->execute(@values);
                $inserted++;
                $count++;
            }
            elsif ( cmpStr( $data_out{$id}, $data_in{$id} ) ) {
                $sth_out_delete->execute($id);
                $sth_out_insert->execute(@values);
                $updated++;
                $count++;
            }

            if ( ($count % $batchcount) == 0 ) {
                $dbh_out->commit;
            }

        }

        $dbh_out->commit;

        my $elapsed = time - $starttime;

        STDOUT->print(
            qq{Update statistics for $source:\n},
            qq{\t$inserted inserts\n},
            qq{\t$updated updates\n},
            qq{\t$deleted deletes\n},
        );

        STDOUT->print( qq{Extraction of $source in $elapsed seconds\n} );

    }

    my $elapsed = time - $time;

    STDOUT->print( qq{Extraction completed in $elapsed seconds\n} );

    return 1;

}

1;

# Local Variables:
#   mode: cperl
#   cperl-indent-level: 4
#   fill-column: 78
# End:
# vim: expandtab shiftwidth=4:
