# Audit methods return empty values when there are no issues with the audit.
# They return lists of error messages when they encounter problems.
# Therefore, these methods have "inverted" success/failure logic.

package EFS::InstallAudit;

use strict;
use warnings;
use English q{-no_match_vars};

use Params::Validate qw( :all );
use List::MoreUtils qw( uniq );
use File::Basename qw( basename );

sub get_required {
    my $class        = shift;
    my $parameter_rs = EFS::DBI->get_resultset( q{Parameter} );
    my $required     = $parameter_rs->configured( q{install_audits_required} ) || q{};
    return sort split qr{[\s,]+}msx, $required;
}

sub get_optional {
    my $class        = shift;
    my $parameter_rs = EFS::DBI->get_resultset( q{Parameter} );
    my $optional     = $parameter_rs->configured( q{install_audits_optional} ) || q{};
    return sort split qr{[\s,]+}msx, $optional;
}

sub get_all {
    my $class = shift;
    return ( $class->get_required, $class->get_optional );
}

sub is_valid {
    my $class = shift;
    my $audit = shift;
    my %audits = map { $_ => 1 } $class->get_all;
    return exists $audits{$audit};
}

sub dependencies {

    my $class = shift;

    my (%args) = validate( @_, {
        install     => { type     => OBJECT,
                         isa      => q{EFS::DBI::Result::Install} },
        filetree    => { type     => OBJECT,
                         isa      => q{EFS::FileTree} },
        fakereadelf => { type     => SCALAR,
                         optional => 1 },
    });

    my $install  = $args{install};
    my $filetree = $args{filetree};

    my $readelf = q{readelf};

    if ( $args{fakereadelf} ) {

        my $parameter_rs = EFS::DBI->get_resultset( q{Parameter} );

        my $dbtype = $parameter_rs->configured( q{database_type} );

        if ( $dbtype ne q{test} ) {
            die qq{The -fakereadelf option is only valid in the test suite.\n};
        }

        $readelf = $args{fakereadelf};

    }

    my $metaproj = $install->metaproj;
    my $project  = $install->project;
    my $release  = $install->release;

    my @filenames =
        sort { lc($a) cmp lc($b) } $filetree->get_filenames;

    my @errors = ();

    my %dependencies =
        map { $_->dependency_object_id => $_ }
            $install->release->dependencies( type => q{runtime} );

    foreach my $filename ( @filenames ) {

        my @file_errors = ();

        # XXX: The testrpath hack is just that: a hack to force the
        # test suite to think those files are executables, when in
        # fact, they are empty files.  See test/bin/fakereadelf.

        # XXX: This is almost certainly not the only file type....
        # We're trying to avoid calling "file" here, and only do so
        # if it looks like a binary, and readelf fails
        if ( $filename !~ m/\.testrpath $/x &&
                 $filetree->get_magic($filename) !~ m{\bELF\b}msx ) {
            next;
        }

        my @readelf_output = qx{ $readelf -d $filename 2>&1 };

        if ( $CHILD_ERROR ) {
            if ( $filename =~ m/\.testrpath $/x ) {
                die(
                    qq{Unconfigured fakereadelf pathname: $filename\n},
                    @readelf_output,
                );
            } else {
                die(
                    qq{Unable to readelf -d $filename:\n},
                    @readelf_output
                );
            }
        }

        chomp @readelf_output;

        foreach my $rpath_line ( @readelf_output ) {

            next unless $rpath_line =~ m/Library \s rpath: \s \[ (.*) \]/x;

            my $dependency_list = $1;

            my @dependencies = split( qr/ : /msx, $dependency_list );

          DEPENDENCY: foreach my $dependency ( @dependencies ) {

                next unless ( $dependency =~ m{ ^ /efs }x );

                # The rest of these all apply even to the same path

                my ($dependency_mpr) =
                    ( $dependency =~ m{ ^ /efs/[^/]+/ ([^/]+/[^/]+/[^/]+) }x );

                if ( $dependency =~ m{ ^ /efs/dev }x ) {
                    push(@file_errors,
                         qq{RPATH items with '/efs/dev/..' } .
                             qq{are not allowed: $dependency});
                }

                if ( $dependency =~ m{ /exec/ }x ) {
                    push(@file_errors,
                         qq{RPATH items with 'exec' } .
                             qq{(rather than '.exec') are } .
                                 qq{not allowed: $dependency});
                }

                if ( not $dependency_mpr ) {
                    push(@file_errors,
                         qq{Unable to parse m/p/r from $dependency});
                    next DEPENDENCY;
                }

                if (  $dependency_mpr eq $release->object_id ) {
                    next DEPENDENCY;
                }

                my ($dependency_metaproj,
                    $dependency_project,
                    $dependency_release) =
                    split( qr{ / }msx, $dependency_mpr );

                my $release_rs = EFS::DBI->get_resultset( q{Release} );

                my $dependency_object = $release_rs->retrieve_release_or_alias(
                        metaproj => $dependency_metaproj,
                        project  => $dependency_project,
                        name     => $dependency_release,
                    );

                if ( not $dependency_object ) {
                    push(@file_errors,
                         qq{RPATH item does not exist: $dependency});
                    next DEPENDENCY;
                }

                if ( not $dependencies{$dependency_mpr} ) {

                    $dependencies{$dependency_mpr} =
                        EFS::CLI::Dependency->define(
                            dependent_metaproj => $metaproj->name,
                            dependent_project  => $project->name,
                            dependent_release  => $release->name,
                            metaproj           => $dependency_metaproj,
                            project            => $dependency_project,
                            release            => $dependency_release,
                            type               => q{runtime},
                        );

                }

            }

        }

        if ( @file_errors ) {
            push( @errors,
                  qq{Filename: $filename\n}, map { "\t$_" } uniq @file_errors );
        }

    }

    return @errors;

}

sub forbidden_files {

    my $class = shift;

    my (%args) = validate( @_, {
        install  => { type => OBJECT,
                      isa  => q{EFS::DBI::Result::Install} },
        filetree => { type => OBJECT,
                      isa  => q{EFS::FileTree} },
    });

    my $filetree = $args{filetree};

    my $ffrule_rs = EFS::DBI->get_resultset( q{ForbiddenFileRule} );

    my @ffrules = $ffrule_rs->all;

    my @filenames =
        sort { lc($a) cmp lc($b) } $filetree->get_filenames;

    my @errors = ();

    foreach my $filename ( @filenames ) {

        my $basename = basename($filename);

        foreach my $ffrule ( @ffrules ) {
            my $regex = $ffrule->regex;
            if ( $basename =~ qr{$regex} ) {  ##  no critic: ExtendedFormatting
                push( @errors,
                      qq{Forbidden file rule '$ffrule' violated by $filename\n} );
            }
        }

    }

    return @errors;

}

1;

# Local Variables:
#   mode: cperl
#   cperl-indent-level: 4
#   fill-column: 78
# End:
# vim: expandtab shiftwidth=4:
