package EFS::RPC;

use strict;
use warnings;
use English;

use EFS::FQDN qw(fqdn);

use Params::Validate qw( :all );
use IO::Socket::SSL;
use Socket;
use POSIX;
use Storable qw(nfreeze thaw);
use List::Util qw(shuffle);
use Readonly;

# Stolen from EFS::CLI
# Should these be the same, and managed in the same place?
Readonly my $RETRY_MAX           => 10;
Readonly my $RETRY_SLEEP         => 6;
Readonly my $TIMEOUT             => 60;

sub execute {

    my $thisclass = shift;

    my (%args) = validate( @_, {
        cell           => { isa  => q{EFS::DBI::Result::Cell}, optional => 1 },
        class          => { type => SCALAR | OBJECT },
        method         => { type => SCALAR },
        args           => { type => ARRAYREF, default  => [], optional => 1 },
        force_remote   => { type => SCALAR, optional => 1 },
        history        => { type => HASHREF, default => {}, optional => 1 },
    });

    my $cell   = $args{cell} || EFS::DBI->get_localcell;
    my $method = $args{method};
    my $args   = $args{args};

    my $class = ref $args{class} || $args{class};

    if ( not $class =~ m/\A EFS::/msx ) {
        die(
            qq{Invalid argument:\n\t},
            qq{\tclass => '$class',\n},
            q{Only methods in EFS::* classes may be },
            qq{executed through EFS::RPC\n},
        );
    }

    if ( not $class->can($method) ) {
        die(
            qq{Invalid arguments:\n\t},
            qq{\tclass => '$class',\n},
            qq{\tmethod => '$method',\n},
            qq{No such method.\n},
        );
    }

    # Specifying history data means we need to force this to go through efsd,
    # so that we get a separate logfile.
    if ( %{ $args{history} } and not $PERLDB ) {
        $args{force_remote}++;
    }

    # Short circuit all this complexity if the localhost supports the
    # requested cell, and just call the method directly.  This allows
    # us to wrap EVERYTHING that we might want to run remotely with an
    # EFS::RPC call, and not worry about inefficiencies.

    my $efsserver_rs = EFS::DBI->get_resultset(q{EFSServer});

    my $efsserver = $efsserver_rs->retrieve_bynames_assert( name => fqdn, );

    if ( not $args{force_remote} and $efsserver->supports_cell($cell) ) {
        return $class->$method( @{ $args } );
    }

    if ( $ENV{EFS_RPC_DEBUG} ) {
        STDOUT->print(
            qq{[DEBUG] Executing $class->$method via EFS::RPC\n}
        );
    }

    my $hosttype = $efsserver->hosttype;

    my @efsservers = shuffle $cell->efsservers_byhosttype($hosttype);

    return $thisclass->execute_on(
        efsservers     => \@efsservers,
        class          => $class,
        method         => $method,
        args           => $args,
        force_remote   => $args{force_remote},
        history        => $args{history},
    );

}

sub execute_on {

    my $thisclass = shift;

    my (%args) = validate( @_, {
        efsservers     => { type     => ARRAYREF },
        class          => { type     => SCALAR },
        method         => { type     => SCALAR },
        args           => { type     => ARRAYREF,
                            default  => [],
                            optional => 1 },
        force_remote   => { type     => SCALAR | UNDEF,
                            optional => 1 },
        history        => { type     => HASHREF,
                            default  => {},
                            optional => 1 },
    });

    my $class  = $args{class};
    my $method = $args{method};
    my $args   = $args{args};

    my @efsservers = @{ $args{efsservers} };

    my $fqdn = fqdn;

    if ( not $args{force_remote} and grep { $fqdn eq $_->name } @efsservers ) {
        return $class->$method(@{ $args });
    }

    my $efsserver_port = get_serverport();

    my $connection = undef;

    my $efsserver_name  = q{};

    my $efsserver_rs = EFS::DBI->get_resultset(q{EFSServer});
    my ($self) = $efsserver_rs->search({ name => $fqdn }) or
        die(qq{Unable to retrieve EFSServer object for $fqdn\n});

SERVER: foreach my $efsserver ( @efsservers ) {

        my $retry = 0;

        $efsserver_name = $efsserver->name;

    PORT: {

            if ( $ENV{EFS_RPC_DEBUG} ) {
                STDOUT->print(
                    qq{[DEBUG] Attempting connection to $efsserver_name },
                    qq{on port $efsserver_port\n},
                );
            }

            $connection = IO::Socket::SSL->new(
                PeerAddr        => $efsserver_name,
                PeerPort        => $efsserver_port,
                Proto           => q{tcp},
                Type            => SOCK_STREAM,
                Timeout         => $TIMEOUT,
                SSL_use_cert    => 1,
                SSL_version     => q{SSLv3},
                SSL_check_crl   => 1,
                SSL_ca_file     => $self->get_ca_certfile,
                SSL_key_file    => $self->get_ssl_keyfile,
                SSL_cert_file   => $self->get_ssl_certfile,
                SSL_crl_file    => $self->get_crl_file,
                SSL_verify_mode => 0x01,
            );

            if ($connection) {
                # debugging message?
                last SERVER;
            }
            else {
                if (    ( $retry < $RETRY_MAX )
                    and ( $ERRNO == ECONNREFUSED or $ERRNO == EINTR ) ) {
                    STDOUT->print(
                        qq{efsd: Connection refused by $efsserver_name:$efsserver_port (retrying...)\n}
                    );
                    sleep($RETRY_SLEEP);
                    $retry++;
                    redo PORT;
                }
                else {
                    STDOUT->print(
                        q{Unable to connect to efsd on server: },
                        qq{$efsserver_name, port: $efsserver_port\n},
                        IO::Socket::SSL::errstr(@_),
                    );
                    next SERVER;
                }
            }

        }    # PORT block

    }    # SERVER block

    if ( not $connection ) {
        die(
            qq{Unable to connect to efsd on any of the these efsservers:\n\t},
            join ( qq{\n\t}, @efsservers ), qq{\n}
        );
    }

    my %client_data = (

        metadata => {
            history  => $args{history},
            class    => $thisclass,
            envvars  => {
                ANSI_COLORS_DISABLED => $ENV{ANSI_COLORS_DISABLED},
            },
        },

        operation => {
            object_class  => $class,
            object_method => $method,
            object_args   => $args,
        },

    );

    my $config = EFS::Config::Server->new;

    if ( $config->get_value( q{security}, q{authtype} ) =~ m{\bGSSAPI\b}msx ) {
        $connection->print( qq{RPC\nGSSAPI\n} );
        EFS::Request->authenticate_gssapi_client($connection,$efsserver_name);
    }
    else {
        $connection->print( qq{RPC\nSSL\n} );
    }

    my $frozen_data = nfreeze( \%client_data );
    $frozen_data = pack( q{N/a*}, $frozen_data );
    $connection->print($frozen_data);

    my $seen_exit_status = undef;

    my $exception = q{};

    my $return = 1;

    while ( defined ( $_ = $connection->getline ) ) {

        chomp;

        ##  no critic: CascadingIfElse
        if (m/\A _EFS_EXCEPTION: ([^\n]*)/msx) {
            $exception .= qq{$1\n};
            next;
        }
        elsif (m/\A _EFS_EXIT_STATUS: \h+ (\d+)/msx) {
            $seen_exit_status = $1;
            last;
        }
        elsif (m/\A _EFS_RETURN_REF: \h+ ([^\n]*)/msx) {
            my $frozen = $1;
            $frozen =~ s/__newline__/\n/gmsx;
            $return = thaw($frozen);
        }
        elsif (m/\A _EFS_RETURN_VALUE: \h+ ([^\n]*)/msx) {
            $return = $1;
            $return =~ s/__newline__/\n/gmx;
        }
        else {
            print qq{$_\n};
        }

    }

    if ($exception) {
        die(
            $exception,
            qq{Error running $class->$method via },
            qq{EFS::RPC on $efsserver_name\n},
        );
    }

    if ( defined $seen_exit_status ) {
        if ($seen_exit_status) {
            die(
                qq{Error running $class->$method via },
                qq{EFS::RPC on $efsserver_name\n}
            );
        }
        else {
            return $return;
        }
    }
    else {
        die(
            qq{Exit status from efs server not received.  Assuming failure...\n}
        );
    }

    return;

}

sub authorize {

    my $class    = shift;
    my $authuser = shift;
    my $authhost = shift;

    if ( $authuser ne q{efsops} ) {
        die qq{Invalid EFS::RPC username '$authuser'\n};
    }

    my $efsserver_rs = EFS::DBI->get_resultset( q{EFSServer} );

    my $client = $efsserver_rs->retrieve_bynames_assert( name => $authhost );
    my $self   = $efsserver_rs->retrieve_bynames_assert( name => fqdn );

    my $client_hosttype = $client->hosttype;
    my $self_hosttype   = $self->hosttype;

    if ( $self_hosttype ne $client_hosttype ) {
        die(
            qq{EFSServer hosttype mismatch:\n},
            qq{Have => $client_hosttype\n},
            qq{Want => $self_hosttype\n},
            qq{EFS::RPC connection hosttypes must match\n},
        );
    }

    return 1;

}

#
# These methods are used exclusively by the test suite.
#
sub test_success {

    my $class = shift;

    my (%args) = validate( @_, {
        scalar   => { type => SCALAR,   optional => 1 },
        arrayref => { type => ARRAYREF, optional => 1 },
        hashref  => { type => HASHREF,  optional => 1 },
        object   => { type => OBJECT,   optional => 1 },
        return   => { type => SCALAR | HASHREF,
                      default => 1, optional => 1 },
    });

    if ( $args{object} && $ENV{EFS_RPC_DEBUG} ) {
        STDOUT->print(
            q{# EFS::RPC->test_success got an object, of type },
            ref $args{object}, qq{\n}
        );
    }

    STDOUT->print(qq{# EFS::RPC->test_success, well.... succeeded\n});

    return $args{return};

}

sub test_failure {
    my $class = shift;
    my $message = shift || q{test_failure};
    die $message;
    return;
}

sub get_serverport {

    Readonly my $PORT_DEFAULT => 17_831;

    if ( $ENV{EFS_SERVERPORT} ) {
        return $ENV{EFS_SERVERPORT};
    }
    elsif ( my @efsd = getservbyname(q{efsd},q{tcp}) ) {
        return $efsd[2];
    }
    else {
        my $config = EFS::Config::Server->new;
        return ( $config->get_value( q{daemon}, q{port} ) || $PORT_DEFAULT );
    }

    ##  TODO: there should be some kind of exception here
    return;

}

1;

# Local Variables:
#   mode: cperl
#   cperl-indent-level: 4
#   fill-column: 78
# End:
# vim: expandtab shiftwidth=4:
