package EFS::Request::RPC;

use strict;
use warnings;
use English;
use feature q{switch};
use Try::Tiny;
use IO::File;

use Moose;
extends q{EFS::Request};

use Storable qw(nfreeze);
use Params::Validate qw( :all );

sub metadata_required {
    return qw( history envvars );
}

sub operation_required {
    return qw( object_class object_method object_args );
}

sub execute {

    my $self = shift;

    my $authuser = $self->authuser;
    my $authhost = $self->authhost;

    my $history = $self->history_obj;

    my $operation = $self->operation;

    my $object_class  = $operation->{object_class};
    my $object_method = $operation->{object_method};
    my $object_args   = $operation->{object_args};

    if ( $history ) {
        push @{ $operation->{object_args} }, (
            history => $history->id,
        );
    }

    if ($PERLDB) {
        EFS::RPC->authorize( $authuser, $authhost );
        $object_class->$object_method(@{ $object_args });
        STDOUT->print( qq{_EFS_EXIT_STATUS: 1\n} );
        return 1;
    }

    my $return = undef;
    my $status = 0;

    my $eval_error = q{};

    try {
        EFS::RPC->authorize( $authuser, $authhost );
        $return = $object_class->$object_method(@{ $object_args });
    }
    catch {
        $eval_error = $_;
    }; 

    if ( $eval_error ) {
        $eval_error =~ s/^/_EFS_EXCEPTION:/gmsx;
        STDOUT->print( $eval_error, qq{_EFS_EXIT_STATUS: 1\n} );
    } else {
        if ( ref $return ) {
            my $frozen = nfreeze($return);
            $frozen =~ s/\n/__newline__/gmsx;
            STDOUT->print( qq{_EFS_RETURN_REF: $frozen\n} );
        } else {
            $return =~ s/\n/__newline__/gmsx;
            STDOUT->print( qq{_EFS_RETURN_VALUE: $return\n} );
        }
        STDOUT->print( qq{_EFS_EXIT_STATUS: 0\n} );
        $status = 1;
    }

    if ( $history ) {
        $history->status( $status );
        $history->update;
    }

    return 1;

}

no Moose;
__PACKAGE__->meta->make_immutable;
1;

# Local Variables:
#   mode: cperl
#   cperl-indent-level: 4
#   fill-column: 78
# End:
# vim: expandtab shiftwidth=4:
