#!/usr/bin/env perl

use strict;
use warnings;
use English;

use Test::More tests => 68;

use lib q{test/lib};
use EFS::Test;

## no critic: RequireExtendedFormatting

my $test   = EFS::Test->new;
my $config = $test->get_config;

my $username  = $test->get_primary_username;
my $groupname = $test->get_primary_groupname;

my $ldc_name      = $config->get_value( q{LocalDevCell}, q{name} );
my $ldc_shortname = $config->get_value( q{LocalDevCell}, q{shortname} );

my $rdc_name      = $config->get_value( q{RemoteDevCell}, q{name} );
my $rdc_shortname = $config->get_value( q{RemoteDevCell}, q{shortname} );

my $lpc_name      = $config->get_value( q{LocalProdCell}, q{name} );
my $lpc_shortname = $config->get_value( q{LocalProdCell}, q{shortname} );

my $mpr_name     = q{testmeta/testproj/testrel};
my $install_root = "/efs/dev/$mpr_name/install";

my $pl32_a = q{x86-32.rhel.5};
my $pl32_b = q{sparc32.sunos.5.10};

my @setup = (
    q{efs display metaproj testmeta},
    q{efs display project testmeta testproj},
    q{efs create release testmeta testproj testrel},
    q{efs create install testmeta testproj testrel common},
    qq{efs create install testmeta testproj testrel $pl32_a},
    qq{efs create install testmeta testproj testrel $pl32_b},
    qq{mkdir /efs/dev/$mpr_name/install/common/bin},
    qq{mkdir /efs/dev/$mpr_name/install/.exec/$pl32_a/lib},
    qq{mkdir /efs/dev/$mpr_name/install/.exec/$pl32_b/lib},
    qq{touch /efs/dev/$mpr_name/install/common/bin/bar},
    qq{touch /efs/dev/$mpr_name/install/.exec/$pl32_a/lib/libfoo.so},
    qq{touch /efs/dev/$mpr_name/install/.exec/$pl32_b/lib/libfoo.so},
);

foreach my $setup (@setup) {
    $test->efs_test_command(
        name    => qq{(setup) $setup},
        command => $setup,
        bail    => 1,
    );
}

my @diststate_legend = (
    q{Legend:},
    q{D: Item is disted and up to date},
    q{P: Item is disted but needs update},
    q{X: Item has not been disted},
);

my @diststate_legend_long = (
    q{Legend:},
    q{-: Item has no checktimes (may be empty)},
    q{D: Item is disted and up to date},
    q{P: Item is disted but needs update},
    q{S: Item is in wrong stage for cell},
    q{X: Item has not been disted},
);

# XXX: Create the dependent release, and try to dist it BEFORE we have
# ever checkpointed it's dependency.

my @commands = (
    q{efs create release testmeta testproj testdep},
    q{efs create install testmeta testproj testdep common},
    q{touch /efs/dev/testmeta/testproj/testdep/install/common/notempty},
    q{efs checkpoint release testmeta testproj testdep},
    q{efs define dependency testmeta testproj testdep testmeta testproj testrel runtime},
);

foreach my $command ( @commands ) {
    $test->efs_test_command(
        command => $command,
    );
}

$test->efs_test_command(
    name    => q{Verify dependency diststate check failure},
    command => q{efs dist release testmeta testproj testdep},
    not     => 1,
    expect  => [
        qq{Dependency $mpr_name has not yet been disted.},
        q{Unable to dist release because the following dependencies},
        q{are not disted to all requested cells.  They must be disted to},
        q{those cells before proceeding any further:},
        qq{$mpr_name},
    ],
);

# The rest of the dependency checks are performed below...

$test->efs_test_command(
    name    => q{perform an initial checkpoint -nolock},
    command => q{efs checkpoint release testmeta testproj testrel -nolock},
    expect  => [
        qq{Performing dependencies audit on install $mpr_name/common},
        qq{Performing forbidden_files audit on install $mpr_name/common},
        qq{Updated initial checktime for install '$mpr_name/common'},
        qq{Performing dependencies audit on install $mpr_name/$pl32_b},
        qq{Performing forbidden_files audit on install $mpr_name/$pl32_b},
        qq{Updated initial checktime for install '$mpr_name/$pl32_b'},
        qq{Performing dependencies audit on install $mpr_name/$pl32_a},
        qq{Performing forbidden_files audit on install $mpr_name/$pl32_a},
        qq{Updated initial checktime for install '$mpr_name/$pl32_a'},
        qq{Successfully completed checkpoint for '$mpr_name'},
    ],
);

$test->efs_test_directory(
    name      => q{verify release 'testrel' not present in /efs/dist},
    directory => qq{/efs/dist/$mpr_name},
    not       => 1,
);

$test->efs_test_command(
    name    => q{check current diststate for 'testrel'},
    command => q{efs diststate release testmeta testproj testrel},
    expect  => [
        qr{NOTE: Release.*has stage dev}ms,
        qr{and therefore has no .prod. diststate.}ms,
        q{Dev environment diststate:},
        qq{Install $ldc_shortname $rdc_shortname},
        qr{=+}ms,
        qq{common X X},
        qr{\S+ \s+ X \s+ X}msx,
        qr{\S+ \s+ X \s+ X}msx,
        qr{-+}ms,
        @diststate_legend,
    ],
);

$test->efs_test_command(
    name    => qq{dist release 'testrel' to cell $ldc_shortname},
    command => qq{efs dist release testmeta testproj testrel -cell $ldc_shortname},
    filter  => qr{to \s+ all \s+ cells}msx,
    expect  => q{Successfully distributed release to all cells},
);

$test->efs_test_command(
    name    => q{check the changed diststate for 'testrel'},
    command => q{efs diststate release testmeta testproj testrel},
    expect  => [
        qr{NOTE:\s+Release.*has\s+stage\s+dev}msx,
        qr{and\s+therefore\s+has\s+no\s+.prod.\s+diststate.}msx,
        q{Dev environment diststate:},
        qr{Install \s+ $ldc_shortname \s+ $rdc_shortname}xms,
        qr{=+}msx,
        qr{common \s+ D \s+ X}xms,
        qr{\S+ \s+ D \s+ X}xms,
        qr{\S+ \s+ D \s+ X}xms,
        qr{-+}msx,
        @diststate_legend,
    ],
);

# Check that diststate respect -cells arg's ordering when given
$test->efs_test_command(
    name    => q{check diststate respects -cells ordering},
    command => qq{efs diststate release testmeta testproj testrel -cells $rdc_shortname,$ldc_shortname},
    expect  => [
        qr{NOTE:\s+Release.*has\s+stage\s+dev}msx,
        qr{and\s+therefore\s+has\s+no\s+'prod'\s+diststate.}msx,
        q{Dev environment diststate:},
        qr{Install \s+ $rdc_shortname \s+ $ldc_shortname}xms,
        qr{=+}msx,
        qr{common \s+ X \s+ D}xms,
        qr{\S+ \s+ X \s+ D}xms,
        qr{\S+ \s+ X \s+ D}xms,
        qr{-+}msx,
        @diststate_legend,
    ],
);


$test->efs_test_command(
    name    => qq{dist release 'testrel' to cell $rdc_shortname},
    command => qq{efs dist release testmeta testproj testrel -cell $rdc_shortname},
    filter  => qr{to \s+ all \s+ cells}msx,
    expect  => q{Successfully distributed release to all cells},
);

$test->efs_test_command(
    name    => q{check the changed diststate for 'testrel'},
    command => q{efs diststate release testmeta testproj testrel},
    expect  => [
        qr{NOTE:\s+Release.*has\s+stage\s+dev}msx,
        qr{and\s+therefore\s+has\s+no\s+'prod'\s+diststate.}msx,
        q{Dev environment is up to date.},
    ],
);

# Add no checkpoint item to legend for -verbose
my @diststate_v_legend = @diststate_legend;
splice ( @diststate_v_legend, 1, 0,
    q{-: Item has no checktimes (may be empty)},
);
splice ( @diststate_v_legend, 4, 0, q{S: Item is in wrong stage for cell}, );

$test->efs_test_command(
    name    => q{check diststate -verbose for 'testrel'},
    command => q{efs diststate release testmeta testproj testrel -verbose},
    expect  => [
        qr{NOTE:\s+Release.*has\s+stage\s+dev}msx,
        qr{and\s+therefore\s+has\s+no\s+'prod'\s+diststate.}msx,
        q{Dev environment diststate:},
        qr{Install \s+ $ldc_shortname \s+ $rdc_shortname}xms,
        qr{=+}msx,
        qr{common \s+ D \s+ D}xms,
        qr{\S+ \s+ D \s+ D}xms,
        qr{\S+ \s+ D \s+ D}xms,
        qr{-+}msx,
        @diststate_v_legend,
    ],
);

$test->efs_test_command(
    name    => q{unlock the release},
    command => q{efs unlock release testmeta testproj testrel},
    expect  => [
        qq{Unlocking directory /efs/dev/$mpr_name/src},
        qq{Unlocking directory /efs/dev/$mpr_name/build},
        qq{Unlocking all installs in release '$mpr_name'},
        qq{Unlocking install $install_root/common},
        qr{Unlocking \s+ install \s+ $install_root/.exec/\S+}msx,
        qr{Unlocking \s+ install \s+ $install_root/.exec/\S+}msx,
        qq{Successfully chowned all installs in $mpr_name},
    ],
);

$test->efs_test_directory(
    name      => q{check src user/group},
    directory => qq{/efs/dev/$mpr_name/src},
    user      => $username,
    group     => $groupname,
);

$test->efs_test_command(
    name    => q{remove common/bin/bar},
    command => qq{rm $install_root/common/bin/bar},
);

$test->efs_test_command(
    name    => q{add common/bin/blah},
    command => qq{touch $install_root/common/bin/blah},
);

$test->efs_test_command(
    name    => q{incremental checkpoint},
    command => q{efs checkpoint release testmeta testproj testrel},
    filter  => qr{Successfully}msx,
    expect  => qq{Successfully completed checkpoint for '$mpr_name'},
);

$test->efs_test_command(
    name    => q{check new diststate},
    command => q{efs diststate release testmeta testproj testrel},
    expect  => [
        qr{NOTE:\s+Release.*has\s+stage\s+dev}msx,
        qr{and\s+therefore\s+has\s+no\s+'prod'\s+diststate.}msx,
        q{Dev environment diststate:},
        qr{Install \s+ $ldc_shortname \s+ $rdc_shortname}xms,
        qr{=+}msx,
        qr{common \s+ P \s+ P}xms,
        qr{\S+ \s+ P \s+ P}xms,
        qr{\S+ \s+ P \s+ P}xms,
        qr{-+}msx,
        @diststate_legend,
    ],

);

$test->efs_test_command(
    name    => q{incremental dist - check /efs/dist/*/*/*/common},
    command => q{efs dist release testmeta testproj testrel --global},
);

$test->efs_test_file(
    name => q{should be no bin/bar},
    file => qq{/efs/dist/$mpr_name/common/bin/bar},
    not  => 1,
);

$test->efs_test_file(
    name => q{should have bin/blah},
    file => qq{/efs/dist/$mpr_name/common/bin/blah},
);

# The following tests are for the NEW prod cell dist mechanism

$test->efs_test_command(
    name    => q{disting a non-prod release to prod cells fails},
    command => q{efs dist release testmeta testproj testrel --celltype prod --global},
    not     => 1,
    expect  => [
        q{Only prod or deprecated releases may be disted to prod cells.},
        qq{Release '$mpr_name' is in stage 'dev'.},
    ],
);

$test->efs_test_command(
    name    => q{setstage release prod},
    command => q{efs setstage release testmeta testproj testrel prod},
    bail    => 1,
);

$test->efs_test_command(
    name    => q{check diststate after setting the stage to prod},
    command => q{efs diststate release testmeta testproj testrel},
    expect  => [
        q{Dev environment is up to date.},
        q{Prod environment diststate:},
        qr{Install \s+ $lpc_shortname}xms,
        qr{=+}msx,
        qr{common \s+ X}xms,
        qr{\S+ \s+ X}xms,
        qr{\S+ \s+ X}xms,
        qr{-+}msx,
        @diststate_legend,
    ],
);

# 00_setup.t leaves testmeta with two userrights defined
# admin for $username, and prod for $username

$test->efs_test_command(
    name    => q{dist with prod userrights rights will work},
    command => q{efs dist release testmeta testproj testrel --dry},
);

$test->efs_test_command(
    command => qq{efs undef userrights testmeta prod $username},
);

$test->efs_test_command(
    name    => q{dist with no prod rights defined},
    command => q{efs dist release testmeta testproj testrel},
    not     => 1,
    expect  => [
        q{Access denied.},
        qq{The 'prod' role is required to dist '$mpr_name'},
        q{Exit status from efs server not received.  Assuming failure...},
    ],
);

$test->efs_test_command(
    command => qq{efs define userrights testmeta prod $username},
    expect  => qq{Granted $username prod rights to testmeta},
);

# [-] verify that users can purge -now a release not yet disted
$test->efs_test_command(
    name    => q{setup testmeta/testproj/purgeme for 1st test},
    command => join ( q{},
        q{efs create release testmeta testproj purgeme && },
        q{efs setstage release testmeta testproj purgeme rejected && },
        q{efs destroy release testmeta testproj purgeme},
    ),
);

$test->efs_test_command(
    name    => q{users can purge -now when release not disted},
    command => q{efs purge release testmeta testproj purgeme -now},
);

$test->efs_test_command(
    name    => q{setup testmeta/testproj/purgeme for 2nd test},
    command => join ( q{},
        q{efs create release testmeta testproj purgeme && },
        q{efs create install testmeta testproj purgeme common && },
        q{touch /efs/dev/testmeta/testproj/purgeme/install/common/foo && },
        q{efs checkpoint release testmeta testproj purgeme && },
        q{efs dist release testmeta testproj purgeme --global && },
        q{efs setstage release testmeta testproj purgeme rejected && },
        q{efs destroy release testmeta testproj purgeme},
    ),
);

$test->efs_test_command(
    name    => q{users can NOT purge -now when release IS disted},
    command => q{efs purge release testmeta testproj purgeme -now},
    not     => 1,
    expect  => [
        q{Only efsops can perform purges with -now once the},
        q{release has been disted anywhere at all.},
    ],
);

$test->efs_test_command(
    run_as  => q{efsops},
    command => q{efs purge release testmeta testproj purgeme -now},
);

# (3) update $mpr_name, so that it is out of date
update_mpr();

# (4) attempt to dist testmeta/testproj/testdep, verify failure
$test->efs_test_command(
    name    => q{Verify dependency diststate check failure},
    command => q{efs dist release testmeta testproj testdep --global},
    not     => 1,
    expect  => [
        qq{Diststate for release dependency $mpr_name:},
        q{Dev environment diststate:},
        qr{Install \s+ $ldc_shortname \s+ $rdc_shortname}xms,
        qr{=+}msx,
        qr{common \s+ P \s+ P}xms,
        qr{\S+ \s+ P \s+ P}xms,
        qr{\S+ \s+ P \s+ P}xms,
        qr{-+}msx,
        @diststate_legend,
        q{Unable to dist release because the following dependencies},
        q{are not disted to all requested cells.  They must be disted to},
        q{those cells before proceeding any further:},
        qq{$mpr_name},
    ],
);

$test->efs_test_command(
    name    => q{Display dependency diststate, in dist},
    command => q{efs diststate dependency testmeta testproj testdep -celltype prod},
    not     => 1,
    expect  => [
        qq{Diststate for release dependency $mpr_name:},
        q{Dev environment diststate:},
        qr{Install \s+ $ldc_shortname}xms,
        qr{=+}msx,
        qr{common \s+ P}xms,
        qr{\S+ \s+ P}xms,
        qr{\S+ \s+ P}xms,
        qr{-+}msx,
        q{Prod environment diststate:},
        qr{Install \s+ $lpc_shortname}xms,
        qr{=+}msx,
        qr{common \s+ X}xms,
        qr{\S+ \s+ X}xms,
        qr{\S+ \s+ X}xms,
        qr{-+}msx,
        @diststate_legend,
        q{One or more dependencies are not up to date.},
    ],
);

# (5) dist $mpr_name
$test->efs_test_command(
    name    => q{Dist dependent release},
    command => q{efs dist release testmeta testproj testrel --global},
);

# (5.1) To avoid another iteration of update_mpr, we'll check the
# -dryrun output here
$test->efs_test_command(
    name    => q{efs dist release -dryrun},
    command => q{efs dist release testmeta testproj testdep --dryrun --global},
    expect  => [
        q{Dev environment diststate:},
        qr{Install \s+ $ldc_shortname \s+ $rdc_shortname}xms,
        qr{^=*}msx,
        qr{common \s+ X \s+ X}xms,
        qr{^-*}msx,
        @diststate_legend,
    ],
);

# (6) redo step (4), verify it works
$test->efs_test_command(
    name    => q{Verify dependency diststate check success},
    command => q{efs dist release testmeta testproj testdep --global},
);

# That's all folks..... Now we cleanup...

foreach my $mpr ( [ q{testmeta}, q{testproj}, q{testdep}, q{rejected} ],
                  [ q{testmeta}, q{testproj}, q{testrel}, q{deprecated} ] ) {
    remove_mpr(@{$mpr});
}

sub update_mpr {

    $test->efs_test_command(
        run_as  => q{efsops},
        name    => qq{unlock $mpr_name again},
        command => q{efs unlock release testmeta testproj testrel},
    );

    $test->efs_test_command(
        name    => q{touch common},
        command => qq{touch $install_root/common},
    );

    $test->efs_test_command(
        run_as  => q{efsops},
        name    => qq{checkpoint $mpr_name again},
        command => q{efs checkpoint release testmeta testproj testrel},
    );

    return;

}

sub remove_mpr {

    my ( $metaproj, $project, $release, $laststage ) = @_;

    $test->efs_test_command(
        run_as  => q{efsops},
        command => qq{efs setstage release $metaproj $project $release $laststage},
    );

    $test->efs_test_command(
        run_as  => q{efsops},
        command => qq{efs destroy release $metaproj $project $release},
    );

    $test->efs_test_command(
        run_as  => q{efsops},
        command => qq{efs recover release $metaproj $project $release},
        expect  => [
            qq{Renamed /efs/dev/$metaproj/$project/.destroyed/$release } .
                qq{to /efs/dev/$metaproj/$project/$release},
            qq{Recovered /efs/dev/$metaproj/$project/$release},
            qq{Renamed $ldc_name:/efs/dist/$metaproj/$project/.destroyed/$release} .
                qq{ to $ldc_name:/efs/dist/$metaproj/$project/$release},
            qq{Recovered $metaproj/$project/$release in /efs/dist in $ldc_name},
            qq{Renamed $rdc_name:/efs/dist/$metaproj/$project/.destroyed/$release} .
                qq{ to $rdc_name:/efs/dist/$metaproj/$project/$release},
            qq{Recovered $metaproj/$project/$release in /efs/dist in $rdc_name},
            qq{Successfully recovered '$metaproj/$project/$release' } .
                q{in all cells},
        ],
    );

    $test->efs_test_file(
        name => q{No .lock file in a recovered release},
        file => qq{/efs/dev/$metaproj/$project/$release/.lock},
        not  => 1,
    );

    $test->efs_test_command(
        run_as  => q{efsops},
        command => qq{efs destroy release $metaproj $project $release},
    );

    $test->efs_test_command(
        run_as  => q{efsops},
        command => qq{efs purge release $metaproj $project $release -now},
    );

    return;

}

exit 0;

