#!/usr/bin/env perl

use strict;
use warnings;
use English;

use Test::More tests => 37;

use lib q{test/lib};
use EFS::Test;

## no critic: RequireExtendedFormatting

my $test   = EFS::Test->new;
my $config = $test->get_config;

my $username  = $test->get_primary_username;
my $groupname = $test->get_primary_groupname;

my $ldc_name = $config->get_value( q{LocalDevCell}, q{name} );

my @setup_one = (
    [ 0, q{efs display metaproj testmeta} ],
    [ 1, q{efs display project testmeta testchild} ],
    [ 0, qq{efs create project testmeta testchild $ldc_name} ],
);

foreach my $setup ( @setup_one ) {
    my ($not,$command) = @{ $setup };
    $test->efs_test_command(
        name    => qq{(setup, phase one) $command},
        not     => $not,
        command => $command,
        bail    => 1,
    );
}

my $parent_path = q{testmeta/testchild/parent};
my $parent_mpr  = q{testmeta testchild parent};
my $child_path  = q{testmeta/testchild/child};
my $child_mpr   = q{testmeta testchild child};

my $setup_one = qq{efs display release $parent_mpr};

$test->efs_test_command(
    name    => qq{(setup, phase one) $setup_one},
    command => $setup_one,
    not     => 1,
    bail    => 1,
);

$setup_one = qq{efs create release $parent_mpr};

$test->efs_test_command(
    name    => qq{(setup, phase one) $setup_one},
    command => $setup_one,
);

$test->efs_test_command(
    command => qq{efs create childrelease $child_mpr parent},
    not     => 1,
    expect  => qq{Parent $parent_path is not locked},
);

my @setup_two = ();

foreach my $install ( qw( x86-64.rhel.5 sparc64.sunos.5.10 ) ) {
    push @setup_two, (
        qq{efs create install $parent_mpr $install},
        map { qq{touch /efs/dev/$parent_path/install/.exec/$install/$_} }
            qw( bin lib ),
    );
}

push @setup_two, (
    qq{efs checkpoint release $parent_mpr},
    qq{efs dist release $parent_mpr},
);

foreach my $setup ( @setup_two ) {
    $test->efs_test_command(
        name    => qq{(setup, phase two) $setup},
        command => $setup,
    );
}

$test->efs_test_command(
    command => qq{efs create childrelease $child_mpr parent},
    not     => 1,
    expect  => qq{Parent $parent_path is not prod},
);

my $setup_three = qq{efs setstage release $parent_mpr prod};

$test->efs_test_command(
    name    => qq{(setup, phase three) $setup_three},
    command => $setup_three,
);

$test->efs_test_command(
    command => qq{efs create childrelease $child_mpr parent},
    filter  => qr{Created child}ms,
    expect  => qq{Created child $child_path, of parent $parent_path},
);

foreach my $name ( qw(x86-64.rhel.5 sparc64.sunos.5.10) ) {

    my $child_dir  = qq{/efs/dev/$child_path/install/.exec/$name};
    my $parent_dir = qq{/efs/dist/$parent_path/.exec/$name};

    $test->efs_test_directory(
        directory => $child_dir,
        user      => 0,
        group     => 0,
        nolink    => 1,
    );

    foreach my $dir ( qw( bin lib ) ) {
        $test->efs_test_link(
            link   => qq{$child_dir/$dir},
            target => qq{$parent_dir/$dir},
        );
    }

}

$test->efs_test_command(
    command => qq{efs display release $child_mpr},
    expect  => [
        qr{Metaproj Project Release Stage}ms,
        qr{=+}msx,
        qr{testmeta testchild child dev}ms,
        qr{Metaproj Project Release Install}ms,
        qr{=+}ms,
        qr{testmeta testchild child sparc64.sunos.5.10}ms,
        qr{testmeta testchild child x86-64.rhel.5}ms,
        qr{Dependencies of 'testmeta/testchild/child'}ms,
        qr{Dependent Release Type}ms,
        qr{=+}ms,
        qr{testmeta/testchild/child testmeta/testchild/parent runtime}ms,
    ],
);

$test->efs_test_command(
    command => qq{efs display release $parent_mpr},
    expect  => [
        qr{Metaproj Project Release Stage}ms,
        qr{=+}ms,
        qr{testmeta testchild parent prod}ms,
        qr{Metaproj Project Release Install}ms,
        qr{=+}ms,
        qr{testmeta testchild parent sparc64.sunos.5.10}ms,
        qr{testmeta testchild parent x86-64.rhel.5}ms,
        qr{Direct Dependents on 'testmeta/testchild/parent'}ms,
        qr{Dependent Release Type}ms,
        qr{=+}ms,
        qr{testmeta/testchild/child testmeta/testchild/parent runtime}ms,
    ],
);

# NOTE: This should NOT unlock the existing installs
$test->efs_test_command(
    command => qq{efs create install $child_mpr x86-64.rhel.5},
);

$test->efs_test_directory(
    directory => qq{/efs/dev/$child_path/install/.exec/x86-64.rhel.5},
    user      => 0,
    group     => 0,
    nolink    => 1,
);

$test->efs_test_command(
    command => qq{efs create install $child_mpr x86-64.rhel.6},
);

$test->efs_test_directory(
    directory => qq{/efs/dev/$child_path/install/.exec/x86-64.rhel.6},
    user      => $username,
    group     => $groupname,
    nolink    => 1,
);

$test->efs_test_command(
    command => q{efs history project testmeta testchild --count 12},
    expect  => [
        qr{Status Authuser Request}ms,
        qr{-+}ms,
        qr{1 $username create project testmeta testchild $ldc_name}ms,
        qr{1 $username create release testmeta testchild parent}ms,
        qr{0 $username create childrelease testmeta testchild child parent}ms,
        qr{1 $username create install testmeta testchild parent x86-64.rhel.5}ms,
        qr{1 $username create install testmeta testchild parent sparc64.sunos.5.10}ms,
        qr{1 $username checkpoint release testmeta testchild parent}ms,
        qr{1 $username dist release testmeta testchild parent}ms,
        qr{0 $username create childrelease testmeta testchild child parent}ms,
        qr{1 $username setstage release testmeta testchild parent prod}ms,
        qr{1 $username create childrelease testmeta testchild child parent}ms,
        qr{1 $username create install testmeta testchild child x86-64.rhel.5}ms,
        qr{1 $username create install testmeta testchild child x86-64.rhel.6}ms,
    ],
);

my @teardown = (
    qq{efs setstage release $child_mpr rejected},
    qq{efs destroy release $child_mpr},
    qq{efs purge release $child_mpr --now},
    qq{efs setstage release $parent_mpr deprecated},
    qq{efs destroy release $parent_mpr},
    qq{efs purge release $parent_mpr --now},
    q{efs destroy project testmeta testchild},
);

foreach my $teardown ( @teardown ) {
    my $run_as = $teardown =~ m{--now}msx ? q{efsops} : q{primary_user};
    $test->efs_test_command(
        run_as  => $run_as,
        name    => qq{(teardown) $teardown},
        command => $teardown,
    );
}

exit 0;
