#!/usr/bin/env perl

use strict;
use warnings;
use English;

use File::Basename qw( dirname );
use Test::More tests => 25;

use blib;
use EFS;

use lib q{test/lib};
use EFS::Test;

my $test = EFS::Test->new;
my $config = $test->get_config;

my $username  = $test->get_primary_username;
my $groupname = $test->get_primary_groupname;

my $ldc_name      = $config->get_value( q{LocalDevCell}, q{name} );
my $ldc_efsserver = $config->get_value( q{LocalDevCell}, q{efsserver} );

my $rdc_name = $config->get_value( q{RemoteDevCell}, q{name} );
my $lpc_name = $config->get_value( q{LocalProdCell}, q{name} );

my $homepath = join q{/}, substr($username,0,1), substr($username,0,2), $username;
my $homedir = qq{/efs/home/$homepath};

$test->efs_test_command(
    run_as  => q{efsops},
    command => qq{efs create home usertoolong $ldc_name},
    not     => 1,
    expect  => [
        q{Invalid home name 'usertoolong'},
        q{Must be =< 8 characters},
        q{Exit status from efs server not received.  Assuming failure...},
    ],
);

$test->efs_test_command(
    run_as  => q{efsops},
    command => qq{efs create home _nosuch_ $ldc_name},
    not     => 1,
    expect  => q{No such username '_nosuch_'},
);

$test->efs_test_command(
    run_as  => q{efsops},
    command => qq{efs create home $username $lpc_name},
    not     => 1,
    expect  => qq{Cell $lpc_name is not a dev cell},
);

$test->efs_test_command(
    run_as  => q{efsops},
    command => qq{efs create home $username $ldc_name},
    filter  => qr{Created \s+ home}msx,
    expect  => qq{Created home $homepath},
);

$test->efs_test_directory(
    directory => $homedir,
    user      => $username,
    group     => $groupname,
    mode      => 02755,
    mask      => 07777,
);

$test->efs_test_command(
    run_as  => q{efsops},
    command => qq{efs move home $username $ldc_name},
    not     => 1,
    expect  =>
        qq{Invalid argument: the target cell $ldc_name matches the home cell $ldc_name},
);

$test->efs_test_command(
    run_as  => q{efsops},
    command => qq{efs move home $username $lpc_name},
    not     => 1,
    expect  => qq{Cell $lpc_name is not a dev cell},
);

my $history_rs = EFS::DBI->get_resultset( q{History} );
my $history    = $history_rs->create({
    command    => q{rebuild namespace},
    request    => q{rebuild namespace dev somecell},
    authuser   => q{efsops},
    efshost    => $ldc_efsserver,
    efsversion => q{test},
    pid        => $PID,
});

$test->efs_test_command(
    run_as  => q{efsops},
    command => qq{efs move home $username $rdc_name},
    not     => 1,
    expect  => [
        q{The following rebuild namespace commands are still active:},
        qr{Id \s+ Authuser \s+ Efshost \s+ Pid \s+ Request}msx,
        qr{ =+ }msx,
        qr{ $PID \s+ rebuild \s+ namespace \s+ dev \s+ somecell }msx,
        q{This command may not be used while active rebuilds are in progress},
    ],
);

$history->delete;

$test->efs_test_command(
    run_as  => q{efsops},
    command => qq{efs move home $username $rdc_name},
);

$test->efs_test_command(
    run_as  => q{efsops},
    command => qq{efs display home $username},
    filter  => qr{\b $username \b}msx,
    expect  => qr{$username \s+ $rdc_name}msx,
);

$test->efs_test_command(
    run_as  => q{efsops},
    command => qq{efs move home $username $ldc_name},
);

$test->efs_test_command(
    run_as  => q{efsops},
    command => qq{efs display home $username},
    filter  => qr{\b $username \b}msx,
    expect  => qr{$username \s+ $ldc_name}msx,
);

$test->efs_test_command(
    run_as  => q{efsops},
    command => qq{efs setattr home $username foo bar},
    expect => [
        qq{For home '$homepath', defined attribute as:},
        q{'foo' => 'bar'},
    ],
);

$test->efs_test_command(
    run_as  => q{efsops},
    command => qq{efs display home $username},
    filter  => qr{\b $username \b}msx,
    expect  => [
        qr{$username \s+ $ldc_name}msx,
        qr{$username \s+ foo \s+ bar}msx,
    ],
);

$test->efs_test_command(
    run_as  => q{efsops},
    command => qq{efs delattr home $username foo},
    expect  => qq{For home '$homepath', deleted attribute 'foo'},
);

$test->efs_test_command(
    run_as  => q{efsops},
    command => qq{efs display home $username},
    filter  => qr{\b $username \b}msx,
    expect  => qr{$username \s+ $ldc_name}msx,
);

my $destroyed_dir  = dirname($homedir) . q{/.destroyed};
my $destroyed_home = qq{$destroyed_dir/$username};

$test->efs_test_command(
    run_as  => q{efsops},
    command => qq{efs destroy home $username},
    expect => [
        qq{Created directory $ldc_name:$destroyed_dir},
        qq{Renamed $ldc_name:$homedir to $ldc_name:$destroyed_home},
        qq{Removed symlink $ldc_name:$homedir},
        qq{Removed symlink $rdc_name:$homedir},
        qq{Destroyed home $homepath},
    ],
);

$test->efs_test_command(
    run_as  => q{efsops},
    command => qq{efs display home $username},
    filter  => qr{\b $username \b}msx,
    expect  => [
        qr{Home \s+ '$homepath' \s+ was \s+ destroyed \s+ on \s+ '\d+-}msx,
        qr{$username \s+ $ldc_name}msx,
    ],
);

$test->efs_test_command(
    run_as  => q{efsops},
    command => qq{efs display home $ldc_name},
    filter  => qr{\b $username \b}msx,
    expect  => qr{$username \s+ $ldc_name \s+ \d+}msx,
);

$test->efs_test_command(
    run_as  => q{efsops},
    command => qq{efs recover home $username},
    expect => [
        qq{Renamed $ldc_name:$destroyed_home to $ldc_name:$homedir},
        qr{Created \s+ symlink \s+ $ldc_name:$homedir}msx,
        qr{Created \s+ symlink \s+ $rdc_name:$homedir}msx,
        qq{Successfully recovered home $homepath},
    ],
);

$test->efs_test_command(
    run_as  => q{efsops},
    command => qq{efs display home $username},
    filter  => qr{\b $username \b}msx,
    expect  => qr{$username \s+ $ldc_name}msx,
);

$test->efs_test_command(
    run_as  => q{efsops},
    command => qq{efs destroy home $username},
    expect => [
        qq{Renamed $ldc_name:$homedir to $ldc_name:$destroyed_home},
        qq{Removed symlink $ldc_name:$homedir},
        qq{Removed symlink $rdc_name:$homedir},
        qq{Destroyed home $homepath},
    ],
);

$test->efs_test_command(
    run_as  => q{efsops},
    command => qq{efs purge home $username},
    expect => [
        qq{Unable to purge $homepath},
        qr{It \s+ was \s+ destroyed \s+ less \s+ than}msx,
    ],
);

$test->efs_test_command(
    run_as  => q{efsops},
    command => qq{efs purge home $username -now},
    expect => [
        qr{Removed \s+ directory \s+ $ldc_name:/efs/home}msx,
        qq{Successfully purged home $username},
    ],
);

$test->efs_test_command(
    run_as  => q{efsops},
    command => qq{efs history home $username -count 14},
    expect => [
        qr{History \s+ ID \s+ StartTime \s+ Duration \s+ Status \s+ Authuser \s+ Request}msx,
        qr{ ^ -+ $ }msx,
        qr{efsops \s+ create \s+ home \s+ $username \s+ $lpc_name}msx,
        qr{efsops \s+ create \s+ home \s+ $username \s+ $ldc_name}msx,
        qr{efsops \s+ move \s+ home \s+ $username \s+ $ldc_name}msx,
        qr{efsops \s+ move \s+ home \s+ $username \s+ $lpc_name}msx,
        qr{efsops \s+ move \s+ home \s+ $username \s+ $rdc_name}msx,
        qr{efsops \s+ move \s+ home \s+ $username \s+ $rdc_name}msx,
        qr{efsops \s+ move \s+ home \s+ $username \s+ $ldc_name}msx,
        qr{efsops \s+ setattr \s+ home \s+ $username \s+ foo \s+ bar}msx,
        qr{efsops \s+ delattr \s+ home \s+ $username \s+ foo}msx,
        qr{efsops \s+ destroy \s+ home \s+ $username}msx,
        qr{efsops \s+ recover \s+ home \s+ $username}msx,
        qr{efsops \s+ destroy \s+ home \s+ $username}msx,
        qr{efsops \s+ purge \s+ home \s+ $username}msx,
        qr{efsops \s+ purge \s+ home \s+ $username \s+ -now}msx,
    ],
);

exit 0;
