#!/usr/bin/env perl

use strict;
use warnings;
use English;

use Test::More tests => 29;

use blib;
use EFS;
use EFS::FQDN qw(fqdn);

use lib q{test/lib};
use EFS::Test;

my $test = EFS::Test->new;
my $config = $test->get_config;

my $username  = $test->get_primary_username;
my $groupname = $test->get_primary_groupname;

my $ldc_name      = $config->get_value( q{LocalDevCell}, q{name} );
my $ldc_shortname = $config->get_value( q{LocalDevCell}, q{shortname} );

my $ldc_efsserver  = $config->get_value( q{LocalDevCell}, q{efsserver} );
my $ldc_fileserver = $config->get_value( q{LocalDevCell}, q{fileserver} );

my $rdc_name      = $config->get_value( q{RemoteDevCell}, q{name} );
my $rdc_shortname = $config->get_value( q{RemoteDevCell}, q{shortname} );

my $lpc_name      = $config->get_value( q{LocalProdCell}, q{name} );
my $lpc_shortname = $config->get_value( q{LocalProdCell}, q{shortname} );

my $root_domain = $config->get_value( q{Global}, q{root_domain} );

my $ownerlist = join q{@}, $username, $root_domain;

my $mp_name = q{testmeta moveme};
my $mp_path = q{testmeta/moveme};

my @setup = (
    qq{efs create metaproj testmeta $username $groupname $ownerlist},
    qq{efs create project $mp_name $ldc_name},
);

foreach my $setup (@setup) {
    $test->efs_test_command(
        run_as  => q{efsops},
        name    => qq{(setup) $setup},
        command => $setup,
        bail    => 1,
    );
}

dist_new_release(q{before});

# Manually set the movelock attribute, and verify that the project is
# "locked"

my $fqdn = fqdn;

$test->efs_test_command(
    run_as  => q{efsops},
    command =>
        qq{efs set_movelock project $mp_name $fqdn $PID $ldc_name $rdc_name},
);

my @commands = (
    qq{efs create release $mp_name bogus},
    qq{efs create project $mp_name $ldc_name},
    qq{efs move project $mp_name $rdc_name},
);

foreach my $command (@commands) {
    $test->efs_test_command(
        command => $command,
        not     => 1,
        expect  => [
            qq{Project '$mp_path' is currently being moved},
            qq{from $ldc_name to $rdc_name},
            qq{by process $PID on $fqdn},
        ],
    );
}

$test->efs_test_command(
    run_as  => q{efsops},
    command => qq{efs clear_movelock project $mp_name},
);

$test->efs_test_command(
    command => qq{efs move project $mp_name $ldc_name},
    not     => 1,
    expect  => [
        qq{Invalid argument: the target cell $ldc_name matches the project cell $ldc_name},
    ],
);

$test->efs_test_command(
    command => qq{efs move project $mp_name $lpc_name},
    not     => 1,
    expect  => [
        qq{Invalid argument: the target cell $lpc_name is not a dev cell},
    ],
);

my $history_rs = EFS::DBI->get_resultset( q{History} );
my $history    = $history_rs->create({
    command    => q{rebuild namespace},
    request    => q{rebuild namespace dev somecell},
    authuser   => q{efsops},
    efshost    => $ldc_efsserver,
    efsversion => q{test},
    pid        => $PID,
});

$test->efs_test_command(
    command => qq{efs move project $mp_name $rdc_name},
    not     => 1,
    expect  => [
        q{The following rebuild namespace commands are still active:},
        qr{Id \s+ Authuser \s+ Efshost \s+ Pid \s+ Request}msx,
        qr{ =+ }msx,
        qr{ $PID \s+ rebuild \s+ namespace \s+ dev \s+ somecell }msx,
        q{This command may not be used while active rebuilds are in progress},
    ],
);

$history->delete;

# Now we test the move for real

$test->efs_test_command(
    command => qq{efs move project $mp_name $rdc_name}
);

$test->efs_test_command(
    command => qq{efs display project $mp_name},
    filter  => qr{$rdc_name}msx,
    expect  => qr{testmeta \s+ moveme \s+ $rdc_name}xms,
);

$test->efs_test_command(
    command => qq{efs move project $mp_name $ldc_name},
);

$test->efs_test_command(
    command => qq{efs display project $mp_name},
    filter  => qr{$ldc_name}msx,
    expect  => qr{testmeta \s+ moveme \s+ $ldc_name}xms,
);

purge_release(q{before});

my @teardown = ( qq{efs destroy project $mp_name} );

foreach my $teardown (@teardown) {
    $test->efs_test_command(
        name    => qq{(teardown) $teardown},
        command => $teardown,
    );
}

exit 0;

sub dist_new_release {

    my $name  = shift;
    my $rlink = $name . q{-rl};

    my @commands = (
        qq{efs create release $mp_name $name},
        qq{efs create install $mp_name $name x86-32.rhel.5},
        qq{efs create install $mp_name $name sparc32.sunos.5.10},
        qq{touch /efs/dev/$mp_path/$name/install/.exec/x86-32.rhel.5/foo},
        qq{touch /efs/dev/$mp_path/$name/install/.exec/sparc32.sunos.5.10/foo},
        qq{touch /efs/dev/$mp_path/$name/build/foo},
        qq{efs checkpoint release $mp_name $name},
        qq{efs dist release $mp_name $name},
        qq{efs create releasealias $mp_name $rlink $name},
        qq{efs dist releasealias $mp_name $rlink},
    );

    foreach my $command (@commands) {
        $test->efs_test_command( command => $command );
    }

    return;

}

sub purge_release {

    my $name  = shift;
    my $rlink = $name . q{-rl};

    my @commands = (
        qq{efs destroy releasealias $mp_name $rlink},
        qq{efs setstage release $mp_name $name rejected},
        qq{efs destroy release $mp_name $name},
        qq{efs purge release $mp_name $name -now},
    );

    foreach my $command (@commands) {
        $test->efs_test_command(
            run_as  => q{efsops},
            command => $command
        );
    }

    return;

}
